import 'dart:io';

import 'package:dio/dio.dart';
import 'package:flutter/material.dart';
import 'package:open_file/open_file.dart';
import 'package:path_provider/path_provider.dart';

class UpdateDialog extends StatefulWidget {
  UpdateDialog(
      {Key? key,
      required this.context,
      required this.titleText,
      required this.contentText,
      required this.rootNavigator,
      this.confirmText,
      required this.allowSkip,
      required this.downloadUrl,
      this.cancelText})
      : super(key: key);

  final BuildContext context;
  final String titleText;
  final String contentText;
  final String? confirmText;
  final String? cancelText;
  final String downloadUrl;
  final bool rootNavigator;
  final bool allowSkip;

  @override
  _UpdateDialogState createState() => _UpdateDialogState();
}

class _UpdateDialogState extends State<UpdateDialog> {
  @override
  Widget build(BuildContext context) {
    return Dialog(
      shape: RoundedRectangleBorder(
        borderRadius: BorderRadius.circular(14),
      ),
      elevation: 0,
      backgroundColor: Colors.white,
      child: changeDialog ? _downloadContent() : _updateContent(),
    );
  }

  bool changeDialog = false;

  Widget _updateContent() {
    return Padding(
      padding: const EdgeInsets.all(14.0),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        children: [
          if (widget.allowSkip)
            Align(
              alignment: Alignment.topRight,
              child: IconButton(
                onPressed: () {
                  _dismiss();
                },
                icon: Icon(Icons.clear_rounded),
              ),
            ),
          Container(
            child: Text(
              '${widget.titleText}',
              textAlign: TextAlign.center,
              style: TextStyle(
                color: Colors.black,
                fontSize: 18,
                fontWeight: FontWeight.bold,
              ),
            ),
            alignment: Alignment.center,
          ),
          SizedBox(
            height: 8,
          ),
          // widget.content,
          Container(
            child: Text(
              '${widget.contentText}',
              textAlign: TextAlign.center,
              style: TextStyle(
                color: Colors.black,
              ),
            ),
            alignment: Alignment.center,
          ),
          SizedBox(
            height: 18,
          ),
          Row(
            mainAxisAlignment: MainAxisAlignment.spaceEvenly,
            children: [
              InkWell(
                onTap: () {
                  setState(() {
                    changeDialog = true;
                  });
                  _downloadApp();
                },
                borderRadius: BorderRadius.circular(14),
                child: Container(
                  decoration: BoxDecoration(
                    color: Colors.black,
                    borderRadius: BorderRadius.circular(
                      14,
                    ),
                  ),
                  padding: EdgeInsets.only(
                    left: 18,
                    right: 18,
                    top: 12,
                    bottom: 12,
                  ),
                  child: Text(
                    '${widget.confirmText}',
                    style: TextStyle(
                      color: Colors.white,
                      fontWeight: FontWeight.bold,
                    ),
                  ),
                ),
              ),
              if (widget.allowSkip)
                InkWell(
                  onTap: () {
                    _dismiss();
                  },
                  borderRadius: BorderRadius.circular(14),
                  child: Container(
                    decoration: BoxDecoration(
                      // color: Colors.white,
                      borderRadius: BorderRadius.circular(
                        14,
                      ),
                    ),
                    padding: EdgeInsets.only(
                      left: 18,
                      right: 18,
                      top: 12,
                      bottom: 12,
                    ),
                    child: Text(
                      '${widget.cancelText}',
                      style: TextStyle(
                        color: Colors.black,
                        fontWeight: FontWeight.bold,
                      ),
                    ),
                  ),
                ),
            ],
          ),
        ],
      ),
    );
  }

  Widget _downloadContent() {
    return Padding(
      padding: const EdgeInsets.all(14.0),
      child: Column(
        mainAxisSize: MainAxisSize.min,
        crossAxisAlignment: CrossAxisAlignment.start,
        children: [
          Text(
            'Downloading...',
            style: TextStyle(
              color: Colors.black,
              fontWeight: FontWeight.bold,
              fontSize: 18,
            ),
          ),
          SizedBox(
            height: 18,
          ),
          Stack(
            fit: StackFit.passthrough,
            children: [
              downloadStarted
                  ? ValueListenableBuilder<double>(
                      valueListenable: progressNotifier,
                      builder: (context, index, _) {
                        return LinearProgressIndicator(
                          value: index,
                          backgroundColor: Colors.grey,
                          valueColor:
                              AlwaysStoppedAnimation<Color>(Colors.black),
                        );
                      },
                    )
                  : LinearProgressIndicator(
                      value: null,
                      backgroundColor: Colors.grey,
                      valueColor: AlwaysStoppedAnimation<Color>(Colors.black),
                    )
            ],
          ),
        ],
      ),
    );
  }

  bool downloadStarted = false;
  ValueNotifier<double> progressNotifier = ValueNotifier(0.0);

  _dismiss() {
    Navigator.of(context, rootNavigator: widget.rootNavigator).pop();
  }

  _downloadApp() async {
    Directory tempDir = await getTemporaryDirectory();
    String tempPath = tempDir.path;

    await Dio().download(
      widget.downloadUrl,
      '$tempPath/app.apk',
      onReceiveProgress: (progress, totalProgress) {
        print('object');
        setState(() {
          downloadStarted = true;
        });
        progressNotifier.value = progress / totalProgress;
        if (progress == totalProgress) {
          _dismiss();
          OpenFile.open('$tempPath/app.apk');
        }
      },
      deleteOnError: true,
    );
  }
}
