library updater;

import 'dart:convert';
import 'package:flutter/material.dart';
import 'package:http/http.dart' as http;
import 'package:package_info_plus/package_info_plus.dart';
import 'package:updater/src/update_dialog.dart';

class Updater {
  Updater({
    required this.url,
    required this.context,
    this.allowSkip = true,
    this.confirmText = 'Update',
    this.cancelText = 'Next Time',
    required this.titleText,
    required this.contentText,
    this.dismissOnTouchOutside = true,
    this.rootNavigator = true,
  }) : assert(url.contains('http') == true, "Update url is not valid!");

  final BuildContext context;
  final String url;
  bool allowSkip;
  bool dismissOnTouchOutside;
  final String? confirmText;
  final String? cancelText;
  final String titleText;
  final String contentText;
  final bool? rootNavigator;

  void check() async {
    var response = await http.get(Uri.parse(url));

    print(response.body);

    var date = jsonDecode(response.body);

    var versionCodeNew = date['versionCode'];
    // String contentTextData = date['contentText'];

    var downloadUrl = date['url'];

    PackageInfo packageInfo = await PackageInfo.fromPlatform();

    int buildNumber = int.parse(packageInfo.buildNumber);

    if (buildNumber < versionCodeNew && downloadUrl.contains('http')) {
      _downloadUrl = downloadUrl;
      _show();
    }
  }

  late String _downloadUrl;

  _show() {
    showDialog(
        context: context,
        barrierDismissible: this.allowSkip,
        builder: (_) {
          return _buildDialog;
        });
  }

  Widget get _buildDialog =>
      WillPopScope(onWillPop: _onWillPop, child: _buildDialogUI());

  Future<bool> _onWillPop() async =>
      allowSkip ? this.dismissOnTouchOutside : allowSkip;

  _buildDialogUI() {
    return UpdateDialog(
      context: this.context,
      titleText: this.titleText,
      contentText: this.contentText,
      confirmText: this.confirmText,
      cancelText: this.cancelText,
      rootNavigator: this.rootNavigator ?? true,
      allowSkip: allowSkip,
      downloadUrl: _downloadUrl,
    );
  }
}
